"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var unist_util_visit_1 = __importDefault(require("unist-util-visit"));
var getSsmlFromMdxAst_1 = __importDefault(require("./getSsmlFromMdxAst"));
var buildSpeechOutputBlock = function (node) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    // @ts-ignore
    var speechOutputAttributes = node.attributes.filter(function (attribute) { return attribute.type === "mdxJsxAttribute"; });
    var id = (_a = speechOutputAttributes.find(function (attribute) { return attribute.name === "id"; })) === null || _a === void 0 ? void 0 : _a.value;
    var lexiconNamesBody = (_e = (_d = (_c = (_b = speechOutputAttributes.find(function (attribute) { return attribute.name === "lexiconNames"; })) === null || _b === void 0 ? void 0 : _b.value) === null || _c === void 0 ? void 0 : _c.data) === null || _d === void 0 ? void 0 : _d.estree) === null || _e === void 0 ? void 0 : _e.body;
    var lexiconNames = (lexiconNamesBody === null || lexiconNamesBody === void 0 ? void 0 : lexiconNamesBody.length) &&
        ((_h = (_g = (_f = lexiconNamesBody[0]) === null || _f === void 0 ? void 0 : _f.expression) === null || _g === void 0 ? void 0 : _g.elements) === null || _h === void 0 ? void 0 : _h.map(function (element) { return element.value; }));
    var ssmlTags = (_j = speechOutputAttributes.find(function (attribute) { return attribute.name === "ssmlTags"; })) === null || _j === void 0 ? void 0 : _j.value;
    var voiceId = (_k = speechOutputAttributes.find(function (attribute) { return attribute.name === "voiceId"; })) === null || _k === void 0 ? void 0 : _k.value;
    if (!id) {
        throw new Error("Missing SpeechOutput ID prop: ".concat(node.value));
    }
    var returnValue = {
        id: id,
        lexiconNames: lexiconNames,
        ssmlTags: ssmlTags,
        voiceId: voiceId,
        text: (0, getSsmlFromMdxAst_1.default)(node),
    };
    return returnValue;
};
var extractSpeechOutputBlocks = function (mdxAst, speechOutputComponentNames, ignoredCharactersRegex) {
    var speechOutputBlocks = [];
    (0, unist_util_visit_1.default)(mdxAst, [
        function (node) {
            return (node.type === "mdxJsxFlowElement" &&
                speechOutputComponentNames.includes(node.name));
        },
    ], function (node) {
        speechOutputBlocks.push(buildSpeechOutputBlock(node));
    });
    if (ignoredCharactersRegex) {
        speechOutputBlocks.forEach(function (block) {
            return (block.text = block.text.replace(new RegExp(ignoredCharactersRegex, "g"), ""));
        });
    }
    return speechOutputBlocks;
};
exports.default = extractSpeechOutputBlocks;
//# sourceMappingURL=extractSpeechOutputBlocks.js.map